@@ -1,9 +1,11 @@
 use crate::{
     accessors,
+    cmdbar::CommandBar,
     components::{
         event_pump, CommandBlocking, CommandInfo, CommitComponent,
-        Component, DrawableComponent, HelpComponent, MsgComponent,
-        ResetComponent, StashMsgComponent,
+        Component, DrawableComponent, HelpComponent,
+        InspectCommitComponent, MsgComponent, ResetComponent,
+        StashMsgComponent,
     },
     keys,
     queue::{Action, InternalEvent, NeedsUpdate, Queue},
@@ -14,16 +16,14 @@ use crate::{
 use anyhow::{anyhow, Result};
 use asyncgit::{sync, AsyncNotification, CWD};
 use crossbeam_channel::Sender;
-use crossterm::event::Event;
-use itertools::Itertools;
-use std::borrow::Cow;
-use strings::commands;
+use crossterm::event::{Event, KeyEvent};
+use strings::{commands, order};
 use tui::{
     backend::Backend,
-    layout::{Alignment, Constraint, Direction, Layout, Rect},
+    layout::{Constraint, Direction, Layout, Rect},
     style::Modifier,
     style::Style,
-    widgets::{Block, Borders, Paragraph, Tabs, Text},
+    widgets::{Block, Borders, Tabs},
     Frame,
 };
 ///
@@ -34,7 +34,8 @@ pub struct App {
     reset: ResetComponent,
     commit: CommitComponent,
     stashmsg_popup: StashMsgComponent,
-    current_commands: Vec<CommandInfo>,
+    inspect_commit_popup: InspectCommitComponent,
+    cmdbar: CommandBar,
     tab: usize,
     revlog: Revlog,
     status_tab: Status,
@@ -59,14 +60,17 @@ impl App {
                 queue.clone(),
                 &theme,
             ),
+            inspect_commit_popup: InspectCommitComponent::new(
+                &queue, sender, &theme,
+            ),
             do_quit: false,
-            current_commands: Vec::new(),
+            cmdbar: CommandBar::new(&theme),
             help: HelpComponent::new(&theme),
             msg: MsgComponent::new(&theme),
             tab: 0,
-            revlog: Revlog::new(&sender, &theme),
-            status_tab: Status::new(&sender, &queue, &theme),
-            stashing_tab: Stashing::new(&sender, &queue, &theme),
+            revlog: Revlog::new(&queue, sender, &theme),
+            status_tab: Status::new(sender, &queue, &theme),
+            stashing_tab: Stashing::new(sender, &queue, &theme),
             stashlist_tab: StashList::new(&queue, &theme),
             queue,
             theme,
@@ -78,17 +82,23 @@ impl App {
         &mut self,
         f: &mut Frame<B>,
     ) -> Result<()> {
+        let fsize = f.size();
+
+        self.cmdbar.refresh_width(fsize.width);
+
         let chunks_main = Layout::default()
             .direction(Direction::Vertical)
             .constraints(
                 [
                     Constraint::Length(2),
                     Constraint::Min(2),
-                    Constraint::Length(1),
+                    Constraint::Length(self.cmdbar.height()),
                 ]
                 .as_ref(),
             )
-            .split(f.size());
+            .split(fsize);
+
+        self.cmdbar.draw(f, chunks_main[2]);
 
         self.draw_tabs(f, chunks_main[0]);
 
@@ -101,13 +111,6 @@ impl App {
             _ => return Err(anyhow!("unknown tab")),
         };
 
-        Self::draw_commands(
-            f,
-            chunks_main[2],
-            self.current_commands.as_slice(),
-            self.theme,
-        );
-
         self.draw_popups(f)?;
 
         Ok(())
@@ -117,7 +120,7 @@ impl App {
     pub fn event(&mut self, ev: Event) -> Result<()> {
         log::trace!("event: {:?}", ev);
 
-        if self.check_quit(ev) {
+        if self.check_quit_key(ev) {
             return Ok(());
         }
 
@@ -128,9 +131,26 @@ impl App {
         } else if let Event::Key(k) = ev {
             let new_flags = match k {
                 keys::TAB_TOGGLE => {
-                    self.toggle_tabs()?;
+                    self.toggle_tabs(false)?;
                     NeedsUpdate::COMMANDS
                 }
+                keys::TAB_TOGGLE_REVERSE
+                | keys::TAB_TOGGLE_REVERSE_WINDOWS => {
+                    self.toggle_tabs(true)?;
+                    NeedsUpdate::COMMANDS
+                }
+                keys::TAB_1
+                | keys::TAB_2
+                | keys::TAB_3
+                | keys::TAB_4 => {
+                    self.switch_tab(k)?;
+                    NeedsUpdate::COMMANDS
+                }
+
+                keys::CMD_BAR_TOGGLE => {
+                    self.cmdbar.toggle_more();
+                    NeedsUpdate::empty()
+                }
 
                 _ => NeedsUpdate::empty(),
             };
@@ -144,8 +164,11 @@ impl App {
         if flags.contains(NeedsUpdate::ALL) {
             self.update()?;
         }
+        //TODO: make this a queue event?
+        //NOTE: set when any tree component changed selection
         if flags.contains(NeedsUpdate::DIFF) {
             self.status_tab.update_diff()?;
+            self.inspect_commit_popup.update_diff()?;
         }
         if flags.contains(NeedsUpdate::COMMANDS) {
             self.update_commands();
@@ -175,19 +198,18 @@ impl App {
 
         self.status_tab.update_git(ev)?;
         self.stashing_tab.update_git(ev)?;
+        self.revlog.update_git(ev)?;
+        self.inspect_commit_popup.update_git(ev)?;
 
-        match ev {
-            AsyncNotification::Diff => (),
-            AsyncNotification::Log => self.revlog.update()?,
-            //TODO: is that needed?
-            AsyncNotification::Status => self.update_commands(),
-        }
+        //TODO: better system for this
+        // can we simply process the queue here and everyone just uses the queue to schedule a cmd update?
+        self.update_commands();
 
         Ok(())
     }
 
     ///
-    pub fn is_quit(&self) -> bool {
+    pub const fn is_quit(&self) -> bool {
         self.do_quit
     }
 
@@ -196,6 +218,7 @@ impl App {
         self.status_tab.anything_pending()
             || self.revlog.any_work_pending()
             || self.stashing_tab.anything_pending()
+            || self.inspect_commit_popup.any_work_pending()
     }
 }
 
@@ -208,6 +231,7 @@ impl App {
             reset,
             commit,
             stashmsg_popup,
+            inspect_commit_popup,
             help,
             revlog,
             status_tab,
@@ -216,7 +240,7 @@ impl App {
         ]
     );
 
-    fn check_quit(&mut self, ev: Event) -> bool {
+    fn check_quit_key(&mut self, ev: Event) -> bool {
         if let Event::Key(e) = ev {
             if let keys::EXIT = e {
                 self.do_quit = true;
@@ -235,15 +259,29 @@ impl App {
         ]
     }
 
-    fn toggle_tabs(&mut self) -> Result<()> {
-        let mut new_tab = self.tab + 1;
-        {
-            let tabs = self.get_tabs();
-            new_tab %= tabs.len();
-        }
+    fn toggle_tabs(&mut self, reverse: bool) -> Result<()> {
+        let tabs_len = self.get_tabs().len();
+        let new_tab = if reverse {
+            self.tab.wrapping_sub(1).min(tabs_len.saturating_sub(1))
+        } else {
+            self.tab.saturating_add(1) % tabs_len
+        };
+
         self.set_tab(new_tab)
     }
 
+    fn switch_tab(&mut self, k: KeyEvent) -> Result<()> {
+        match k {
+            keys::TAB_1 => self.set_tab(0)?,
+            keys::TAB_2 => self.set_tab(1)?,
+            keys::TAB_3 => self.set_tab(2)?,
+            keys::TAB_4 => self.set_tab(3)?,
+            _ => (),
+        }
+
+        Ok(())
+    }
+
     fn set_tab(&mut self, tab: usize) -> Result<()> {
         let tabs = self.get_tabs();
         for (i, t) in tabs.into_iter().enumerate() {
@@ -261,8 +299,7 @@ impl App {
 
     fn update_commands(&mut self) {
         self.help.set_cmds(self.commands(true));
-        self.current_commands = self.commands(false);
-        self.current_commands.sort_by_key(|e| e.order);
+        self.cmdbar.set_cmds(self.commands(false));
     }
 
     fn process_queue(&mut self) -> Result<NeedsUpdate> {
@@ -329,6 +366,10 @@ impl App {
                 self.stashmsg_popup.show()?
             }
             InternalEvent::TabSwitch => self.set_tab(0)?,
+            InternalEvent::InspectCommit(id) => {
+                self.inspect_commit_popup.open(id)?;
+                flags.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS)
+            }
         };
 
         Ok(flags)
@@ -352,7 +393,15 @@ impl App {
                 true,
                 !self.any_popup_visible(),
             )
-            .hidden(),
+            .order(order::NAV),
+        );
+        res.push(
+            CommandInfo::new(
+                commands::TOGGLE_TABS_DIRECT,
+                true,
+                !self.any_popup_visible(),
+            )
+            .order(order::NAV),
         );
 
         res.push(
@@ -372,19 +421,31 @@ impl App {
             || self.help.is_visible()
             || self.reset.is_visible()
             || self.msg.is_visible()
+            || self.stashmsg_popup.is_visible()
+            || self.inspect_commit_popup.is_visible()
     }
 
     fn draw_popups<B: Backend>(
         &mut self,
         f: &mut Frame<B>,
     ) -> Result<()> {
-        let size = f.size();
+        let size = Layout::default()
+            .direction(Direction::Vertical)
+            .constraints(
+                [
+                    Constraint::Min(1),
+                    Constraint::Length(self.cmdbar.height()),
+                ]
+                .as_ref(),
+            )
+            .split(f.size())[0];
 
         self.commit.draw(f, size)?;
         self.stashmsg_popup.draw(f, size)?;
         self.reset.draw(f, size)?;
         self.help.draw(f, size)?;
         self.msg.draw(f, size)?;
+        self.inspect_commit_popup.draw(f, size)?;
 
         Ok(())
     }
@@ -398,7 +459,7 @@ impl App {
                     strings::TAB_STATUS,
                     strings::TAB_LOG,
                     strings::TAB_STASHING,
-                    "Stashes",
+                    strings::TAB_STASHES,
                 ])
                 .style(Style::default())
                 .highlight_style(
@@ -411,36 +472,4 @@ impl App {
             r,
         );
     }
-
-    fn draw_commands<B: Backend>(
-        f: &mut Frame<B>,
-        r: Rect,
-        cmds: &[CommandInfo],
-        theme: Theme,
-    ) {
-        let splitter = Text::Styled(
-            Cow::from(strings::CMD_SPLITTER),
-            Style::default(),
-        );
-
-        let texts = cmds
-            .iter()
-            .filter_map(|c| {
-                if c.show_in_quickbar() {
-                    Some(Text::Styled(
-                        Cow::from(c.text.name),
-                        theme.toolbar(c.enabled),
-                    ))
-                } else {
-                    None
-                }
-            })
-            .collect::<Vec<_>>();
-
-        f.render_widget(
-            Paragraph::new(texts.iter().intersperse(&splitter))
-                .alignment(Alignment::Left),
-            r,
-        );
-    }
 }
