@@ -17,6 +17,22 @@ use std::path::Path;
 use strings::commands;
 use tui::{backend::Backend, layout::Rect, Frame};
 
+/// macro to simplify running code that might return Err.
+/// It will show a popup in that case
+#[macro_export]
+macro_rules! try_or_popup {
+    ($self:ident, $msg:literal, $e:expr) => {
+        if let Err(err) = $e {
+            $self.queue.borrow_mut().push_back(
+                InternalEvent::ShowErrorMsg(format!(
+                    "{}\n{}",
+                    $msg, err
+                )),
+            );
+        }
+    };
+}
+
 ///
 pub struct ChangesComponent {
     files: FileTreeComponent,
@@ -37,7 +53,7 @@ impl ChangesComponent {
             files: FileTreeComponent::new(
                 title,
                 focus,
-                queue.clone(),
+                Some(queue.clone()),
                 theme,
             ),
             is_working_dir,
@@ -59,7 +75,8 @@ impl ChangesComponent {
 
     ///
     pub fn focus_select(&mut self, focus: bool) {
-        self.files.focus_select(focus)
+        self.files.focus(focus);
+        self.files.show_selection(focus);
     }
 
     /// returns true if list is empty
@@ -76,17 +93,15 @@ impl ChangesComponent {
         if let Some(tree_item) = self.selection() {
             if self.is_working_dir {
                 if let FileTreeItemKind::File(i) = tree_item.kind {
-                    if let Some(status) = i.status {
-                        let path = Path::new(i.path.as_str());
-                        match status {
-                            StatusItemType::Deleted => {
-                                sync::stage_addremoved(CWD, path)?
-                            }
-                            _ => sync::stage_add_file(CWD, path)?,
-                        };
+                    let path = Path::new(i.path.as_str());
+                    match i.status {
+                        StatusItemType::Deleted => {
+                            sync::stage_addremoved(CWD, path)?
+                        }
+                        _ => sync::stage_add_file(CWD, path)?,
+                    };
 
-                        return Ok(true);
-                    }
+                    return Ok(true);
                 } else {
                     //TODO: check if we can handle the one file case with it aswell
                     sync::stage_add_all(
@@ -97,8 +112,7 @@ impl ChangesComponent {
                     return Ok(true);
                 }
             } else {
-                let path =
-                    Path::new(tree_item.info.full_path.as_str());
+                let path = tree_item.info.full_path.as_str();
                 sync::reset_stage(CWD, path)?;
                 return Ok(true);
             }
@@ -107,6 +121,26 @@ impl ChangesComponent {
         Ok(false)
     }
 
+    fn index_add_all(&mut self) -> Result<()> {
+        sync::stage_add_all(CWD, "*")?;
+
+        self.queue
+            .borrow_mut()
+            .push_back(InternalEvent::Update(NeedsUpdate::ALL));
+
+        Ok(())
+    }
+
+    fn stage_remove_all(&mut self) -> Result<()> {
+        sync::reset_stage(CWD, "*")?;
+
+        self.queue
+            .borrow_mut()
+            .push_back(InternalEvent::Update(NeedsUpdate::ALL));
+
+        Ok(())
+    }
+
     fn dispatch_reset_workdir(&mut self) -> bool {
         if let Some(tree_item) = self.selection() {
             let is_folder =
@@ -160,6 +194,11 @@ impl Component for ChangesComponent {
         let some_selection = self.selection().is_some();
 
         if self.is_working_dir {
+            out.push(CommandInfo::new(
+                commands::STAGE_ALL,
+                some_selection,
+                self.focused(),
+            ));
             out.push(CommandInfo::new(
                 commands::STAGE_ITEM,
                 some_selection,
@@ -181,6 +220,11 @@ impl Component for ChangesComponent {
                 some_selection,
                 self.focused(),
             ));
+            out.push(CommandInfo::new(
+                commands::UNSTAGE_ALL,
+                some_selection,
+                self.focused(),
+            ));
             out.push(
                 CommandInfo::new(
                     commands::COMMIT_OPEN,
@@ -212,15 +256,33 @@ impl Component for ChangesComponent {
                         Ok(true)
                     }
                     keys::STATUS_STAGE_FILE => {
-                        if self.index_add_remove()? {
-                            self.queue.borrow_mut().push_back(
-                                InternalEvent::Update(
-                                    NeedsUpdate::ALL,
-                                ),
-                            );
-                        }
+                        try_or_popup!(
+                            self,
+                            "staging error:",
+                            self.index_add_remove()
+                        );
+
+                        self.queue.borrow_mut().push_back(
+                            InternalEvent::Update(NeedsUpdate::ALL),
+                        );
+
                         Ok(true)
                     }
+
+                    keys::STATUS_STAGE_ALL if !self.is_empty() => {
+                        if self.is_working_dir {
+                            try_or_popup!(
+                                self,
+                                "staging error:",
+                                self.index_add_all()
+                            );
+                        } else {
+                            self.stage_remove_all()?;
+                        }
+
+                        Ok(true)
+                    }
+
                     keys::STATUS_RESET_FILE
                         if self.is_working_dir =>
                     {
