@@ -30,7 +30,7 @@ pub struct CommitList {
     count_total: usize,
     items: ItemBatch,
     scroll_state: (Instant, f32),
-    tags: Tags,
+    tags: Option<Tags>,
     current_size: (u16, u16),
     scroll_top: usize,
     theme: Theme,
@@ -44,7 +44,7 @@ impl CommitList {
             selection: 0,
             count_total: 0,
             scroll_state: (Instant::now(), 0_f32),
-            tags: Tags::new(),
+            tags: None,
             current_size: (0, 0),
             scroll_top: 0,
             theme: *theme,
@@ -58,12 +58,12 @@ impl CommitList {
     }
 
     ///
-    pub fn selection(&self) -> usize {
+    pub const fn selection(&self) -> usize {
         self.selection
     }
 
     ///
-    pub fn current_size(&self) -> (u16, u16) {
+    pub const fn current_size(&self) -> (u16, u16) {
         self.current_size
     }
 
@@ -73,23 +73,37 @@ impl CommitList {
     }
 
     ///
+    #[allow(clippy::missing_const_for_fn)]
     pub fn selection_max(&self) -> usize {
         self.count_total.saturating_sub(1)
     }
 
     ///
-    pub fn tags(&self) -> &Tags {
-        &self.tags
+    pub fn tags(&self) -> Option<&Tags> {
+        self.tags.as_ref()
+    }
+
+    ///
+    pub fn has_tags(&self) -> bool {
+        self.tags.is_some()
+    }
+
+    ///
+    pub fn clear(&mut self) {
+        self.tags = None;
+        self.items.clear();
     }
 
     ///
     pub fn set_tags(&mut self, tags: Tags) {
-        self.tags = tags;
+        self.tags = Some(tags);
     }
 
     ///
     pub fn selected_entry(&self) -> Option<&LogEntry> {
-        self.items.iter().nth(self.selection)
+        self.items.iter().nth(
+            self.selection.saturating_sub(self.items.index_offset()),
+        )
     }
 
     fn move_selection(&mut self, scroll: ScrollType) -> Result<bool> {
@@ -197,7 +211,7 @@ impl CommitList {
             } else {
                 String::from("")
             }),
-            theme.tab(true).bg(theme.text(true, selected).bg),
+            theme.tags(selected),
         ));
 
         txt.push(splitter);
@@ -222,7 +236,9 @@ impl CommitList {
             .take(height)
             .enumerate()
         {
-            let tag = if let Some(tags) = self.tags.get(&e.id) {
+            let tags = if let Some(tags) =
+                self.tags.as_ref().and_then(|t| t.get(&e.id))
+            {
                 Some(tags.join(" "))
             } else {
                 None
@@ -232,7 +248,7 @@ impl CommitList {
                 e,
                 idx + self.scroll_top == selection,
                 &mut txt,
-                tag,
+                tags,
                 &self.theme,
             );
         }
@@ -240,6 +256,7 @@ impl CommitList {
         txt
     }
 
+    #[allow(clippy::missing_const_for_fn)]
     fn relative_selection(&self) -> usize {
         self.selection.saturating_sub(self.items.index_offset())
     }
