@@ -1,9 +1,10 @@
-use super::{
-    visibility_blocking, CommandBlocking, CommandInfo, Component,
-    DrawableComponent,
-};
 use crate::{
-    components::dialog_paragraph, strings, ui, ui::style::Theme,
+    components::{
+        popup_paragraph, visibility_blocking, CommandBlocking,
+        CommandInfo, Component, DrawableComponent,
+    },
+    strings, ui,
+    ui::style::Theme,
 };
 use anyhow::Result;
 use crossterm::event::{Event, KeyCode};
@@ -75,7 +76,12 @@ impl DrawableComponent for TextInputComponent {
             let area = ui::centered_rect(60, 20, f.size());
             f.render_widget(Clear, area);
             f.render_widget(
-                dialog_paragraph(self.title.as_str(), txt.iter()),
+                popup_paragraph(
+                    self.title.as_str(),
+                    txt.iter(),
+                    &self.theme,
+                    true,
+                ),
                 area,
             );
         }
