@@ -14,10 +14,17 @@ const fn with_mod(
     KeyEvent { code, modifiers }
 }
 
+pub const TAB_1: KeyEvent = no_mod(KeyCode::Char('1'));
+pub const TAB_2: KeyEvent = no_mod(KeyCode::Char('2'));
+pub const TAB_3: KeyEvent = no_mod(KeyCode::Char('3'));
+pub const TAB_4: KeyEvent = no_mod(KeyCode::Char('4'));
 pub const TAB_TOGGLE: KeyEvent = no_mod(KeyCode::Tab);
 pub const TAB_TOGGLE_REVERSE: KeyEvent = no_mod(KeyCode::BackTab);
-pub const FOCUS_WORKDIR: KeyEvent = no_mod(KeyCode::Char('1'));
-pub const FOCUS_STAGE: KeyEvent = no_mod(KeyCode::Char('2'));
+//TODO: https://github.com/extrawurst/gitui/issues/112
+pub const TAB_TOGGLE_REVERSE_WINDOWS: KeyEvent =
+    with_mod(KeyCode::BackTab, KeyModifiers::SHIFT);
+pub const FOCUS_WORKDIR: KeyEvent = no_mod(KeyCode::Char('w'));
+pub const FOCUS_STAGE: KeyEvent = no_mod(KeyCode::Char('s'));
 pub const FOCUS_RIGHT: KeyEvent = no_mod(KeyCode::Right);
 pub const FOCUS_LEFT: KeyEvent = no_mod(KeyCode::Left);
 pub const EXIT: KeyEvent =
@@ -40,6 +47,7 @@ pub const SHIFT_DOWN: KeyEvent =
     with_mod(KeyCode::Down, KeyModifiers::SHIFT);
 pub const ENTER: KeyEvent = no_mod(KeyCode::Enter);
 pub const STATUS_STAGE_FILE: KeyEvent = no_mod(KeyCode::Enter);
+pub const STATUS_STAGE_ALL: KeyEvent = no_mod(KeyCode::Char('a'));
 pub const STATUS_RESET_FILE: KeyEvent =
     with_mod(KeyCode::Char('D'), KeyModifiers::SHIFT);
 pub const STATUS_IGNORE_FILE: KeyEvent = no_mod(KeyCode::Char('i'));
@@ -52,3 +60,4 @@ pub const STASH_APPLY: KeyEvent = no_mod(KeyCode::Enter);
 pub const STASH_DROP: KeyEvent =
     with_mod(KeyCode::Char('D'), KeyModifiers::SHIFT);
 pub const CMD_BAR_TOGGLE: KeyEvent = no_mod(KeyCode::Char('.'));
+pub const LOG_COMMIT_DETAILS: KeyEvent = no_mod(KeyCode::Enter);
