@@ -4,11 +4,13 @@
 //https://github.com/crossterm-rs/crossterm/issues/432
 #![allow(clippy::cargo::multiple_crate_versions)]
 #![deny(clippy::pedantic)]
+#![deny(clippy::nursery)]
 #![deny(clippy::result_unwrap_used)]
 #![deny(clippy::panic)]
 #![allow(clippy::module_name_repetitions)]
 
 mod app;
+mod cmdbar;
 mod components;
 mod keys;
 mod poll;
@@ -23,6 +25,10 @@ use crate::{app::App, poll::QueueEvent};
 use anyhow::{anyhow, Result};
 use asyncgit::AsyncNotification;
 use backtrace::Backtrace;
+use clap::{
+    crate_authors, crate_description, crate_name, crate_version,
+    App as ClapApp, Arg,
+};
 use crossbeam_channel::{tick, unbounded, Receiver, Select};
 use crossterm::{
     terminal::{
@@ -53,10 +59,10 @@ static TICK_INTERVAL: Duration = Duration::from_secs(5);
 static SPINNER_INTERVAL: Duration = Duration::from_millis(50);
 
 fn main() -> Result<()> {
-    setup_logging()?;
+    process_cmdline()?;
 
-    if invalid_path() {
-        eprintln!("invalid git path\nplease run gitui inside of a git repository");
+    if !valid_path()? {
+        eprintln!("invalid path\nplease run gitui inside of a non-bare git repository");
         return Ok(());
     }
 
@@ -145,8 +151,9 @@ fn draw<B: Backend>(
     })
 }
 
-fn invalid_path() -> bool {
-    !asyncgit::is_repo(asyncgit::CWD)
+fn valid_path() -> Result<bool> {
+    Ok(asyncgit::sync::is_repo(asyncgit::CWD)
+        && !asyncgit::sync::is_bare_repo(asyncgit::CWD)?)
 }
 
 fn select_event(
@@ -205,15 +212,46 @@ fn get_app_config_path() -> Result<PathBuf> {
 }
 
 fn setup_logging() -> Result<()> {
-    if env::var("GITUI_LOGGING").is_ok() {
-        let mut path = get_app_config_path()?;
-        path.push("gitui.log");
+    let mut path = get_app_config_path()?;
+    path.push("gitui.log");
 
-        let _ = WriteLogger::init(
-            LevelFilter::Trace,
-            Config::default(),
-            File::create(path)?,
+    let _ = WriteLogger::init(
+        LevelFilter::Trace,
+        Config::default(),
+        File::create(path)?,
+    );
+
+    Ok(())
+}
+
+fn process_cmdline() -> Result<()> {
+    let app = ClapApp::new(crate_name!())
+        .author(crate_authors!())
+        .version(crate_version!())
+        .about(crate_description!())
+        .arg(
+            Arg::with_name("logging")
+                .help("Stores logging output into a cache directory")
+                .short("l")
+                .long("logging"),
+        )
+        .arg(
+            Arg::with_name("directory")
+                .help("Set the working directory")
+                .short("d")
+                .long("directory")
+                .takes_value(true),
         );
+
+    let arg_matches = app.get_matches();
+    if arg_matches.is_present("logging") {
+        setup_logging()?;
+    }
+
+    if arg_matches.is_present("directory") {
+        let directory =
+            arg_matches.value_of("directory").unwrap_or(".");
+        env::set_current_dir(directory)?;
     }
 
     Ok(())
